<?php

namespace Drupal\meta_conversion_api\Services;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use FacebookAds\Api;
use FacebookAds\Object\ServerSide\Content;
use FacebookAds\Object\ServerSide\CustomData;
use FacebookAds\Object\ServerSide\Event;
use FacebookAds\Object\ServerSide\EventRequest;
use FacebookAds\Object\ServerSide\UserData;

/**
 * Services class for MetaClient.
 *
 * @package Drupal\meta_conversion_api
 */
class MetaClient {

  /**
   * Facebook API.
   *
   * @var \FacebookAds\Api
   */
  protected Api $api;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected ConfigFactory $configFactory;

  /**
   * Logger Factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactory
   */
  protected LoggerChannelFactoryInterface $loggerFactory;

  /**
   * MetaClient constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   An instance of Config Factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   An instance of Logger Channel Factory.
   */
  public function __construct(ConfigFactory $config_factory, LoggerChannelFactoryInterface $logger_factory) {
    $this->config = $config_factory->get('meta_conversion_api.settings');
    $this->loggerFactory = $logger_factory;
    $this->api = Api::init(
      NULL,
      NULL,
      $this->config->get('meta_conversion_api_access_token')
    );
  }

  /**
   * Sends request to the Facebook client.
   *
   * @param array $data
   *   The event data.
   * @param string|null $test_event_code
   *   Meta test event code.
   */
  public function sendRequest(array $data, string $test_event_code = NULL): void {
    // Check to see if the required fields are available, if not exit out of function early.
    $input_event_data = $this->contains_required_data($data['event_data'], $this->return_required_event_fields());

    if (!($input_event_data)) {
      $this->loggerFactory->get('meta_conversion_api')->error('Missing Meta Conversion required fields');
      return;
    }

    // Check to see pixel id and the access token has been saved, if not exist out of function early.
    $access_token = $this->config->get('meta_conversion_api_access_token');
    $pixel_id = $this->config->get('meta_conversion_api_pixel_id');
    if (empty($pixel_id) || empty($access_token)) {
      $this->loggerFactory->get('meta_conversion_api')->error('Please add the required Meta Conversion API Details');
      return;
    }

    // If the test event code isn't passed directly into the function
    // then try using configuration.
    if (empty($test_event_code)) {
      $test_event_code = $this->config->get('meta_conversion_api_test_event_code');
    }

    // Load all the data needed for the request.
    $user_data = $this->userData($data['user_data']);
    $content = $this->content($data['product_data']);
    $custom_data = $this->customData($content, $data);

    // Combine all the data into an event.
    $event = $this->event($data['event_data'], $user_data, $custom_data);

    $events[] = $event;

    // Prepare request.
    $request = (new EventRequest($pixel_id))
      ->setEvents($events)
      ->setTestEventCode($test_event_code);

    try {
      // Send request over to Meta.
      $request->execute();
    }
    catch (\Exception $exception) {
      $this->loggerFactory->get('meta_conversion_api')->error("Meta failed to send the request ", (array) $exception);
    }

  }

  /**
   * Contains the Content object.
   *
   * @param array $user_data
   *  The Meta User data array.
   *
   * @return UserData
   *  Returns the User
   */
  protected function userData($user_data) : UserData {
    return (new UserData())
      ->setFirstName(hash('sha256', $user_data['first_name']))
      ->setLastName(hash('sha256', $user_data['surname']))
      ->setEmail(hash('sha256', $user_data['email']))
      ->setPhone(hash('sha256', $user_data['phone_number']))
      ->setClientIpAddress($_SERVER['REMOTE_ADDR'])
      ->setClientUserAgent($_SERVER['HTTP_USER_AGENT']);
  }

  /**
   * Contains the Content object.
   *
   * @param array $product_data
   *   The Meta Product data array.
   *
   * @return Content
   *   Returns Content object.
   */
  protected function content($product_data) : Content {
    return (new Content())
      ->setProductId($product_data['product_id'])
      ->setDescription($product_data['payment_type'])
      ->setItemPrice($product_data['price'])
      ->setQuantity(1);
  }

  /**
   * Contains the CustomData object.
   *
   * @param Content $content
   *   The Meta Content array.
   *
   * @param array $data
   *   The full Data array.
   *
   * @return CustomData
   *   Returns CustomData object.
   */
  protected function customData(Content $content, array $data) : CustomData {
    return (new CustomData())
      ->setCurrency('GBP')
      ->setContents(array($content))
      ->setContentName($data['event_data']['content_id'])
      ->setContentType($data['event_data']['content_type'])
      ->setValue($data['product_data']['price']);
  }

  /**
   * Contains the Event object.
   *
   * @param array $event_data
   *   The Meta event data array.
   * @param array $user_data
   *   The Meta user data array.
   * @param array $custom_data
   *   The Meta custom data array.
   *
   * @return Event
   *   Returns the Event object.
   */
  protected function event($event_data, UserData $user_data, CustomData $custom_data) : Event {
    return (new Event())
      ->setEventId($event_data['event_id'])
      ->setEventName($event_data['event_name'])
      ->setEventSourceUrl($event_data['event_source_url'])
      ->setEventTime($event_data['event_time'])
      ->setActionSource($event_data['action_source'])
      ->setUserData($user_data)
      ->setCustomData($custom_data);
  }

  /**
   * Checks to see if the input data sent to the Meta Conversion API contains the required data.
   *
   * @param array $input_data
   *   All input data being sent to the Meta Conversion APIU module.
   * @param array $required_fields
   *   Required fields determined by the Meta Conversion API module as necessary.
   *
   * @return bool
   *   If there is a required field missing in the input values then return false and log the error.
   */
  protected function contains_required_data(array $input_data, array $required_fields) : bool {
    // Loop through all the required fields - if there is an empty then flag in error logs.
    foreach($required_fields as $field) {
      if (empty($input_data[$field])) {
        $this->loggerFactory->get('meta_conversion_api')->error('Meta Conversion API required fields missing - @field', ['@field' => $field]);
        return FALSE;
      }
    }
    return TRUE;
  }

  /**
   * Return all the required event fields.
   *
   * @return array
   *   Return a list of required event values.
   */
  protected function return_required_event_fields() : array {
    return [
        'event_id',
        'event_name',
        'event_source_url',
        'event_time',
        'action_source',
        'content_type',
        'content_id',
        'currency',
        'value',
      ];
  }
}
